==> recline-0.0.31.tgz: package/forms.js <==
/** @module recline/forms */


var recline = require('./index'),
	  ev = require('events'),
	  error = require('./errors'),
		_ = require("underscore"),
		field_types = require('./dep/fields'),
		validations = require('./dep/validations');

/**
	Sets the dbName, i.e "recline-forms"
	@param {string} string name of database
*/
exports.dbName = function(string) {
	this.dbname = string;
}

/**
	Parses an array of fields and complies them
	into template snippets for later rendering
	<br/>
	<b>It expects the array items to be objects:</b><br/>
	<p>
	<pre>
        "name": "email",
        "label": "Enter your email address",
        "type": "email",
        "primary": true,
        "required": true 
	</pre>
	</p>
	
	@param {array} fields array of fields
*/
exports.parseFields = function(fields /*array*/){
	
	var _fields = [];
	var l = fields.length;
	for(var i = 0;i<l; i++) {
			var f = {};
			var field = fields[i];
			
			function parseArray(type, field){
				var length = field.options.length;
				var array = [];
				for(var j = 0;j<length; j++) {							
					var clone = _.clone(field);
					if(!field.value){
						// Removing temporarily as I don't want to set them anymore
						if(j == 0){ var checked = false }else{ var checked = false }
						
					}else { /* Insert validation logic here. */ }
					
					clone._id = clone.name+'%'+j;
					if(type == "cboption") clone.name = clone.name+'%'+j;	
					
					if(type == "label-cb" || type == "label-r"){
						clone.name = clone.name+'%'+j;
						var complied = _.template(field_types.lookup("label"));
					}else
					{
						var complied = _.template(field_types.lookup(type));
					}			
					
					array.push(complied(_.extend(clone, { label:clone.options[j], value: clone.options[j], checked: checked })));
					
				}			
				return array;
			}

			
			switch(field.type){
				case 'radios':
						options = parseArray('roption', field)
						labels = parseArray('label-r', field);
						options = _.zip(options, labels);
						f.field = options;
					break;
				case 'select':
					options = parseArray('soption', field)
					var selectopen = _.template(field_types.lookup(field.type));
					selectopen = selectopen(field);
					var selectclose = "</select>"
					f.field = {
						"openTag" : selectopen,
						"options" : options,
						"closeTag" : selectclose  
					};
					break
				case 'checkboxes':
					options = parseArray('cboption', field);
					labels = parseArray('label-cb', field);
					options = _.zip(options, labels);
					f.field = options;
					break;
				default:
					var complied = _.template(field_types.lookup(field.type));
					f.field = complied(field);
					break;
			}
			
	
			complied = _.template(field_types.lookup("label"));
			f.label = complied(field);
		
		  if(field.invalid) { 
				complied = _.template(field_types.lookup("invalid"));
				f.invalid = complied(field);
			}
		
			_fields.push(f);
	}	
	//console.log(_fields);
	return _fields;
		
} 
/**
	Sets up the form based on the form id
	It emits 'ready' event once the form 
	is ready to be manipulated
	
	@param {string} id id of the form
	@return {Forms} Form
*/
exports.id = function(id){
	var self = this;
	if(typeof id != 'string') {
		throw new Error("id must be string");
	}
	
	var
			fClient = new Forms(),
		 	connection,
		 	dbname = this.dbname,
		 	data,
			rev,
		 	db,
			template;
			
	
	
	recline.createdb(this.dbname, function(err, ok){
		if(ok){
			connection = true;
			db = recline[dbname];
			fClient.emit('ready');
		} 
	});
	
	fClient.__defineGetter__('id', function(){
		return id;
	});
	
	fClient.__defineGetter__('rev', function(){
		return rev;
	});
	
	fClient.__defineSetter__('rev', function(r){
		rev = r;
	});
	
	fClient.__defineGetter__('connected', function(){
		return connection;
	});
	
	fClient.__defineGetter__('data', function(){
		return data;
	});
	
	fClient.__defineSetter__('data', function(d){
		data = d;
	});
	
	fClient.__defineGetter__('db', function(){
		return db;
	});
	
	fClient.__defineSetter__('template', function(t){
		template = t;
	});
	
	fClient.__defineGetter__('template', function(){
		return template;
	});
	
	fClient._get = function(){
		db.getDoc(id, function(er,res){
			if(er) { fClient.create() }
			else{ 
				data = res;
				fClient.rev = data._rev;	
				fClient.emit('data', er, res);  }
		});
	}
	
	/**
		Returns args in a familar pattern
		return {object} $ callback, switch, id
		
	*/
	fClient._parseArgs = function(args){
		var $ = {};
		for(var key in args){
				// Annoying fix just for jsdoc
				if(_.isFunction(args[key])) $.cb = args[key];
				if(_.isBoolean(args[key])) 	$["switch"] = args[key];
				if(_.isString(args[key])) 	$.id = args[key];
		}
		return $;
	}
	
	return fClient;
	
}

/**
	The main forms Klass
	@class Forms
*/
var Forms = exports.Forms = function(){
	ev.EventEmitter.call(this);
}

Forms.super_ = ev.EventEmitter;
Forms.prototype = Object.create(ev.EventEmitter.prototype, {
    constructor: {
        value: Forms,
        enumerable: false
    }
});


/*
	Emitter
	TODO: It might not be a bad idea to change this to a call back?
*/

/**
	Gets the form based on the id
	returns either the form or 404
	if it doesn't exist
*/
Forms.prototype.getOr404 = function(){
	var self = this;
	this.db.getDoc(this.id, function(er,res){
		if(er) { self.emit('data', 404); }
		else{ 
			self.data = res;
			self.rev = self.data._rev;
			self.emit('data', er, res);  }
	});
}

/**
	Validates the form fields against predeined conditions
	@param {object} submissionBody
	@return {boolean} isValid
*/
Forms.prototype.validateFields = function(submissionBody /* object */){
	var self = this;
	//self.types = _.map(this.data.fields, function(field){ if(field.required) return [field.name, field.type] });
	self.isValid = true;
	//var l = self.types.length;
	for(var key in submissionBody) {
		
					var orginalKey = key;
					if(key.indexOf("%")>0){
						key = key.split("%")[0];
					}
					
					var pos = self.data.meta[key].pos;
					var validation = validations.lookup(self.data.meta[key].type);
					var r = validation[0];
					r = new RegExp(r);
					
					self.data.fields[pos].value = submissionBody[orginalKey];
					if(!r.test(submissionBody[orginalKey]))
					{
						self.data.fields[pos].invalid = validation[1];
						self.isValid = false;
					} 

			
		}
	
	return self.isValid;
}

/**
	Saves a document to the document store
	@param {object} submissionBody
	@return {boolean}
*/
Forms.prototype.saveDocument = function(submissionBody /* object */){
	var self = this;
	for(var key in submissionBody) {
		if(typeof submissionBody[key] != form.data.meta[key].dataType) throw new error.NotAcceptable("data did not validate metatest");
	}
	this.db.saveDoc(_.extend({ "form_parent" : this.id, "type" : "form_child" }, submissionBody))
	return true;
}

/**
	Gets a template based on a regular patther "formid_tmpl"
	if no template name if given
	
*/
Forms.prototype.getTemplate = function(){
	// ADD check to see if template already attached?
	args = this._parseArgs(arguments);
	var self = this;
	var _name;
	var _cb;
	var _switch;
	
	args.id ? _name = args.id :  _name = this.id + "_tmpl"; 
	args.cb ? _cb = args.cb : _cb = false;
	// Annoying fix for jsdoc
	args["switch"] ? _switch = args["switch"] : _switch = false;
	
	this.db.getAttachment(this.id, _name, function(er,data){
		self.template = data;
		_cb && _cb(er,data);
	}) 
	
}

/**
	Attachs a template to a form document
*/
Forms.prototype.attachTemplate = function() {
	var fs = require('fs');
	args = this._parseArgs(arguments);
	var self = this;
	if(typeof args.id === "undefined"){
		throw new Error("no name for template provided");
	}
	var _name = args.id;
	var path = recline.tempDir + _name;
	
	this.db.saveAttachment(  path, 
													 this.id, 
												 { name: this.id + "_tmpl", 
													 contentType: "text/html",
													 rev: this.rev },
												 function(er,res){
													 if(er) args.cb && args.cb(er,null);
													 args.cb && args.cb(null,res);
												});
	
}

==> recline-0.0.32.tgz: package/forms.js <==
/** @module recline/forms */


var recline = require('./index'),
	  ev = require('events'),
	  error = require('./errors'),
		_ = require("underscore"),
		field_types = require('./dep/fields'),
		validations = require('./dep/validations');

/**
	Sets the dbName, i.e "recline-forms"
	@param {string} string name of database
*/
exports.dbName = function(string) {
	this.dbname = string;
}

/**
	Parses an array of fields and complies them
	into template snippets for later rendering
	<br/>
	<b>It expects the array items to be objects:</b><br/>
	<p>
	<pre>
        "name": "email",
        "label": "Enter your email address",
        "type": "email",
        "primary": true,
        "required": true 
	</pre>
	</p>
	
	@param {array} fields array of fields
*/
exports.parseFields = function(fields /*array*/){
	
	var _fields = [];
	var l = fields.length;
	for(var i = 0;i<l; i++) {
			var f = {};
			var field = fields[i];
			
			function parseArray(type, field){
				var length = field.options.length;
				var array = [];
				for(var j = 0;j<length; j++) {							
					var clone = _.clone(field);
					if(!field.value){
						// Removing temporarily as I don't want to set them anymore
						if(j == 0){ var checked = false }else{ var checked = false }
						
					}else { /* Insert validation logic here. */ }
					
					clone._id = clone.name+'%'+j;
					if(type == "cboption") clone.name = clone.name+'%'+j;	
					
					if(type == "label-cb" || type == "label-r"){
						clone.name = clone.name+'%'+j;
						var complied = _.template(field_types.lookup("label"));
					}else
					{
						var complied = _.template(field_types.lookup(type));
					}			
					
					array.push(complied(_.extend(clone, { label:clone.options[j], value: clone.options[j], checked: checked })));
					
				}			
				return array;
			}

			
			switch(field.type){
				case 'radios':
						options = parseArray('roption', field)
						labels = parseArray('label-r', field);
						options = _.zip(options, labels);
						f.field = options;
					break;
				case 'select':
					options = parseArray('soption', field)
					var selectopen = _.template(field_types.lookup(field.type));
					selectopen = selectopen(field);
					var selectclose = "</select>"
					f.field = {
						"openTag" : selectopen,
						"options" : options,
						"closeTag" : selectclose  
					};
					break
				case 'checkboxes':
					options = parseArray('cboption', field);
					labels = parseArray('label-cb', field);
					options = _.zip(options, labels);
					f.field = options;
					break;
				default:
					var complied = _.template(field_types.lookup(field.type));
					f.field = complied(field);
					break;
			}
			
	
			complied = _.template(field_types.lookup("label"));
			f.label = complied(field);
		
		  if(field.invalid) { 
				complied = _.template(field_types.lookup("invalid"));
				f.invalid = complied(field);
			}
		
			_fields.push(f);
	}	
	//console.log(_fields);
	return _fields;
		
} 
/**
	Sets up the form based on the form id
	It emits 'ready' event once the form 
	is ready to be manipulated
	
	@param {string} id id of the form
	@return {Forms} Form
*/
exports.id = function(id){
	var self = this;
	if(typeof id != 'string') {
		throw new Error("id must be string");
	}
	
	var
			fClient = new Forms(),
		 	connection,
		 	dbname = this.dbname,
		 	data,
			rev,
		 	db,
			template;
			
	
	
	recline.createdb(this.dbname, function(err, ok){
		if(ok){
			connection = true;
			db = recline[dbname];
			fClient.emit('ready');
		} 
	});
	
	fClient.__defineGetter__('id', function(){
		return id;
	});
	
	fClient.__defineGetter__('rev', function(){
		return rev;
	});
	
	fClient.__defineSetter__('rev', function(r){
		rev = r;
	});
	
	fClient.__defineGetter__('connected', function(){
		return connection;
	});
	
	fClient.__defineGetter__('data', function(){
		return data;
	});
	
	fClient.__defineSetter__('data', function(d){
		data = d;
	});
	
	fClient.__defineGetter__('db', function(){
		return db;
	});
	
	fClient.__defineSetter__('template', function(t){
		template = t;
	});
	
	fClient.__defineGetter__('template', function(){
		return template;
	});
	
	fClient._get = function(){
		db.getDoc(id, function(er,res){
			if(er) { fClient.create() }
			else{ 
				data = res;
				fClient.rev = data._rev;	
				fClient.emit('data', er, res);  }
		});
	}
	
	/**
		Returns args in a familar pattern
		return {object} $ callback, switch, id
		
	*/
	fClient._parseArgs = function(args){
		var $ = {};
		for(var key in args){
				// Annoying fix just for jsdoc
				if(_.isFunction(args[key])) $.cb = args[key];
				if(_.isBoolean(args[key])) 	$["switch"] = args[key];
				if(_.isString(args[key])) 	$.id = args[key];
		}
		return $;
	}
	
	return fClient;
	
}

/**
	The main forms Klass
	@class Forms
*/
var Forms = exports.Forms = function(){
	ev.EventEmitter.call(this);
}

Forms.super_ = ev.EventEmitter;
Forms.prototype = Object.create(ev.EventEmitter.prototype, {
    constructor: {
        value: Forms,
        enumerable: false
    }
});


/*
	Emitter
	TODO: It might not be a bad idea to change this to a call back?
*/

/**
	Gets the form based on the id
	returns either the form or 404
	if it doesn't exist
*/
Forms.prototype.getOr404 = function(){
	var self = this;
	this.db.getDoc(this.id, function(er,res){
		if(er) { self.emit('data', 404); }
		else{ 
			self.data = res;
			self.rev = self.data._rev;
			self.emit('data', er, res);  }
	});
}

/**
	Validates the form fields against predeined conditions
	@param {object} submissionBody
	@return {boolean} isValid
*/
Forms.prototype.validateFields = function(submissionBody /* object */){
	var self = this;
	//self.types = _.map(this.data.fields, function(field){ if(field.required) return [field.name, field.type] });
	self.isValid = true;
	//var l = self.types.length;
	for(var key in self.data.meta) {
			if(self.data.meta[key].required){
				
				
				var pos = self.data.meta[key].pos;
				var validation = validations.lookup(self.data.meta[key].type);
				var r = validation[0];
				r = new RegExp(r);
				
				if(self.data.meta[key].type == "checkboxes"){
					var l = self.data.fields[pos].options.length;
					for(var i = 0;i<l; i++){
						if(typeof submissionBody[key +"%"+i] != "undefined") key = key +"%"+i;
					}
				}
				
				
				self.data.fields[pos].value = submissionBody[key];
				if(!r.test(submissionBody[key]))
				{
					self.data.fields[pos].invalid = validation[1];
					self.isValid = false;
				}
			}
		}
	
	return self.isValid;
}

/**
	Saves a document to the document store
	@param {object} submissionBody
	@return {boolean}
*/
Forms.prototype.saveDocument = function(submissionBody /* object */){
	var self = this;
	for(var key in submissionBody) {
		if(typeof submissionBody[key] != form.data.meta[key].dataType) throw new error.NotAcceptable("data did not validate metatest");
	}
	this.db.saveDoc(_.extend({ "form_parent" : this.id, "type" : "form_child" }, submissionBody))
	return true;
}

/**
	Gets a template based on a regular patther "formid_tmpl"
	if no template name if given
	
*/
Forms.prototype.getTemplate = function(){
	// ADD check to see if template already attached?
	args = this._parseArgs(arguments);
	var self = this;
	var _name;
	var _cb;
	var _switch;
	
	args.id ? _name = args.id :  _name = this.id + "_tmpl"; 
	args.cb ? _cb = args.cb : _cb = false;
	// Annoying fix for jsdoc
	args["switch"] ? _switch = args["switch"] : _switch = false;
	
	this.db.getAttachment(this.id, _name, function(er,data){
		self.template = data;
		_cb && _cb(er,data);
	}) 
	
}

/**
	Attachs a template to a form document
*/
Forms.prototype.attachTemplate = function() {
	var fs = require('fs');
	args = this._parseArgs(arguments);
	var self = this;
	if(typeof args.id === "undefined"){
		throw new Error("no name for template provided");
	}
	var _name = args.id;
	var path = recline.tempDir + _name;
	
	this.db.saveAttachment(  path, 
													 this.id, 
												 { name: this.id + "_tmpl", 
													 contentType: "text/html",
													 rev: this.rev },
												 function(er,res){
													 if(er) args.cb && args.cb(er,null);
													 args.cb && args.cb(null,res);
												});
	
}
